/****************************************************************************************
|  Description: Operating system header file
|    File Name: os.h
|
|----------------------------------------------------------------------------------------
|                          C O P Y R I G H T
|----------------------------------------------------------------------------------------
|     Copyright (c) 2012 by Feaser     http://www.feaser.com     All rights reserved
|
|   This software is the property of Feaser. The source and compiled code may only be 
|        used on the Olimex EM32G880F128-STK board (http://www.olimex.com/dev).
| 
|      This copyright notice may not be removed from the source code nor changed.
|
|   This software has been carefully tested, but is not guaranteed for any particular
| purpose. The author does not offer any warranties and does not guarantee the accuracy,
|   adequacy, or completeness of the software and is not responsible for any errors or
|              omissions or the results obtained from use of the software.
|
|***************************************************************************************/
#ifndef OS_H
#define OS_H


/****************************************************************************************
* Configuration
****************************************************************************************/
/* The current version supports up to 16 tasks. This configurable specifies the
 * number of tasks your application intends to use. Its value must be non-zero 
 * and smaller or equal to 16. Lowering the number of supported tasks, reduces
 * the RAM required by the operating system.
 */
#define OS_MAX_TASKS         (16)

/* Set this configurable to how many times the OsTimeTick function is called. It
 * is not directly used by the operating system itself, but enables the user
 * to quickly convert time to time ticks. For example, delay a task for half a
 * second could be achieved using OsTaskDelay(OS_TICKS_PER_SEC/2).
 */
#define OS_TICKS_PER_SEC     (1000)

/* During critical sections of the code, typically when accessing shared global
 * variables, the interrupts are temporarily disabled. This is achieved by placing
 * the critical code sections between the macros OS_ENTER_CRITICAL and 
 * OS_EXIT_CRITICAL.
 */
#define OS_ENTER_CRITICAL()  HwInterruptDisable()

/* During critical sections of the code, typically when accessing shared global
 * variables, the interrupts are temporarily disabled. This is achieved by placing
 * the critical code sections between the macros OS_ENTER_CRITICAL and 
 * OS_EXIT_CRITICAL.
 */
#define OS_EXIT_CRITICAL()   HwInterruptRestore()


/****************************************************************************************
* External function prototypes
****************************************************************************************/
extern void HwInterruptDisable(void);
extern void HwInterruptRestore(void);


/****************************************************************************************
* Defines
****************************************************************************************/
/* Boolean true value used by the operating system functions */
#define OS_TRUE              (1)

/* Boolean false value used by the operating system functions */
#define OS_FALSE             (0)

/* Identifies the active task in operating system functions requiring a handle parameter
 */
#define OS_PRIO_SELF         (OS_MAX_TASKS)


/****************************************************************************************
* Function prototypes
****************************************************************************************/
void          OsInit(void);
void          OsTimeTick(void);
void          OsStart(void);
unsigned char OsTaskCreate(void (*task)(void), unsigned char prio, unsigned short period);
unsigned char OsTaskDelete(unsigned char prio);
unsigned char OsTaskDelay(unsigned char prio, unsigned short delay);
unsigned char OsTaskReady(unsigned char prio);


/****************************************************************************************
* Hook function prototypes
****************************************************************************************/
extern void   HwOsTimeTickInit(void);


/****************************************************************************************
* Configuration check
****************************************************************************************/
#ifndef OS_MAX_TASKS
#error  "OS.H, Configuration macro OS_MAX_TASKS is missing."
#endif

#if     OS_MAX_TASKS < 1
#error  "OS.H, Max number of tasks should be at least 1."
#endif

#if     OS_MAX_TASKS > 16
#error  "OS.H, Max number of tasks should be no greater than 16."
#endif

#ifndef OS_TICKS_PER_SEC
#error  "OS.H, Configuration macro OS_TICKS_PER_SEC is missing."
#endif

#if     OS_TICKS_PER_SEC < 1
#error  "OS.H, Time tick frequency should be > 1."
#endif

#endif /* OS_H */
/********************************* end of os.h *****************************************/


