/****************************************************************************************
|  Description: software application source file
|    File Name: app.c
|
|----------------------------------------------------------------------------------------
|                          C O P Y R I G H T
|----------------------------------------------------------------------------------------
|     Copyright (c) 2012 by Feaser     http://www.feaser.com     All rights reserved
|
|   This software is the property of Feaser. The source and compiled code may only be 
|        used on the Olimex EM32G880F128-STK board (http://www.olimex.com/dev).
| 
|      This copyright notice may not be removed from the source code nor changed.
|
|   This software has been carefully tested, but is not guaranteed for any particular
| purpose. The author does not offer any warranties and does not guarantee the accuracy,
|   adequacy, or completeness of the software and is not responsible for any errors or
|              omissions or the results obtained from use of the software.
|
|***************************************************************************************/

/****************************************************************************************
* Include files
****************************************************************************************/
#include "header.h"                                  /* generic header file            */


/****************************************************************************************
* Macro definitions
****************************************************************************************/
#define BUTTON_BEEP_TASK_PRIO     (tskIDLE_PRIORITY + 2)
#define BUTTON_BEEP_TASK_PERIOD   ((portTickType)100/portTICK_RATE_MS)


/****************************************************************************************
* Function prototypes
****************************************************************************************/
static void ButtonBeepTask(void *pvParameters);
static void TrackButtonPresses(tButtonId id);


/****************************************************************************************
** NAME:           AppCtrlTask
** PARAMETER:      pvParameters: pointer to task parameters
** RETURN VALUE:   none
** DESCRIPTION:    Application control task.
**
****************************************************************************************/
void AppCtrlTask(void *pvParameters)
{
  unsigned char ledOn = 0;

  /* perform one time initializations */
  LcdWrite("TESTAPP");
  LcdNumber(0);

  /* create the button beep task */
  xTaskCreate(ButtonBeepTask, (signed char *) "BtnBeepTask", configMINIMAL_STACK_SIZE, 
              NULL, BUTTON_BEEP_TASK_PRIO, NULL);
  /* enter task body */
  for( ;; )
  {
    /* periodically toggle the LEDs */
    ledOn = (ledOn == FALSE) ? TRUE : FALSE;
    LedSet(LED_UP, ledOn);
    LedSet(LED_DOWN, ledOn);
    LedSet(LED_RIGHT, ledOn);
    LedSet(LED_LEFT, ledOn);
    /* activate this task periodically */
    vTaskDelay(APP_CTRL_TASK_PERIOD);
  }
} /*** end of AppCtrlTask ***/


/****************************************************************************************
** NAME:           ButtonBeepTask
** PARAMETER:      pvParameters: pointer to task parameters
** RETURN VALUE:   none
** DESCRIPTION:    Button beep task that generates a beep on the buzzer each time a 
**                 button got pressed.
**
****************************************************************************************/
static void ButtonBeepTask(void *pvParameters)
{
  unsigned char butPressedLast[4] = { FALSE, FALSE, FALSE, FALSE };
  unsigned char butPressed[4];
  unsigned char butCnt;
  const tButtonId buttons[4] = { BUT1, BUT2, BUT3, BUT4 };

  for (;;)
  {
    for (butCnt=0; butCnt<4; butCnt++)
    {
      /* read current state of button 1 */
      butPressed[butCnt] = ButtonGet(buttons[butCnt]);
      /* check if the button got pressed */
      if ( (butPressed[butCnt] == TRUE) && (butPressed[butCnt] != butPressedLast[butCnt]) )
      {
        /* keep track of how often the button was pressed */
        TrackButtonPresses(buttons[butCnt]);
        /* generate beep */
        BuzzerOn(100);
      }
      /* store for change detection */
      butPressedLast[butCnt] = butPressed[butCnt];
    }
    /* activate this task periodically */
    vTaskDelay(BUTTON_BEEP_TASK_PERIOD);
  }
} /*** end of ButtonBeepTask ***/


/****************************************************************************************
** NAME:           TrackButtonPresses
** PARAMETER:      id button id of the button that got pressed.
** RETURN VALUE:   none
** DESCRIPTION:    Keeps track of each button press by updating a counter on the LCD.
**
****************************************************************************************/
static void TrackButtonPresses(tButtonId id)
{
  static unsigned char pressedCounter[4] = { 0, 0, 0, 0 };
  unsigned short number = 0;
  unsigned char buttonCnt;

  /* update the counter for this button */
  switch (id)
  {
    case BUT1:
      if (++pressedCounter[3] == 10)
      {
        pressedCounter[3] = 0;
      }
      break;
    case BUT2:
      if (++pressedCounter[2] == 10)
      {
        pressedCounter[2] = 0;
      }
      break;
    case BUT3:
      if (++pressedCounter[1] == 10)
      {
        pressedCounter[1] = 0;
      }
      break;
    case BUT4:
      if (++pressedCounter[0] == 10)
      {
        pressedCounter[0] = 0;
      }
      break;
    default:
      break;
  }
  /* update the lcd */
  number += pressedCounter[0];
  number += pressedCounter[1] * 10;
  number += pressedCounter[2] * 100;
  number += pressedCounter[3] * 1000;
  LcdNumber(number);
} /*** end of TrackButtonPresses ***/


/****************************************************************************************
** NAME:           vApplicationTickHook
** PARAMETER:      none
** RETURN VALUE:   none
** DESCRIPTION:    FreeRTOS callback function that is called each time tick increment.
**
****************************************************************************************/
void vApplicationTickHook(void)
{
  /* run the periodic buzzer update */
  BuzzerUpdate();
} /*** end of vApplicationTickHook ***/


/*********************************** end of app.c **************************************/
