/****************************************************************************************
|  Description: hardware interface source file
|    File Name: hw.c
|
|----------------------------------------------------------------------------------------
|                          C O P Y R I G H T
|----------------------------------------------------------------------------------------
|     Copyright (c) 2012 by Feaser     http://www.feaser.com     All rights reserved
|
|   This software is the property of Feaser. The source and compiled code may only be 
|        used on the Olimex EM32G880F128-STK board (http://www.olimex.com/dev).
| 
|      This copyright notice may not be removed from the source code nor changed.
|
|   This software has been carefully tested, but is not guaranteed for any particular
| purpose. The author does not offer any warranties and does not guarantee the accuracy,
|   adequacy, or completeness of the software and is not responsible for any errors or
|              omissions or the results obtained from use of the software.
|
|****************************************************************************************/

/****************************************************************************************
* Include files
****************************************************************************************/
#include "header.h"                                   /* generic header file           */
#include "efm32.h"                                    /* EFM32 registers               */
#include "efm32_chip.h"                               /* EFM32 chip initialization     */
#include "efm32_cmu.h"                                /* EFM32 clock management        */


/****************************************************************************************
* External function prototypes
****************************************************************************************/
extern void reset_handler(void);                      /* implemented in cstart.s       */
extern void xPortPendSVHandler(void);                 /* implemented in port.c         */
extern void xPortSysTickHandler(void);                /* implemented in port.c         */
extern void vPortSVCHandler( void );                  /* implemented in port.c         */


/****************************************************************************************
* External data declarations
****************************************************************************************/
extern unsigned long __stack_end__;                   /* stack end address (memory.x)  */


/****************************************************************************************
* Type definitions
****************************************************************************************/
typedef union
{
  void (*func)(void);                                 /* for ISR function pointers     */
  unsigned long ptr;                                  /* for stack pointer entry       */
}tIsrFunc;                                            /* type for vector table entries */


/****************************************************************************************
** NAME:           HwInit
** PARAMETER:      none
** RETURN VALUE:   none
** DESCRIPTION:    Initializes the hardware interface.
**
****************************************************************************************/
void HwInit(void)
{
  /* initialize the system and its clocks */
  SystemInit();
  /* handle chip errate workarounds */
  CHIP_Init();
  /* configure NVIC priority grouping as:
   *  4 bits for pre-emption priority
   *  0 bits for subpriority 
   */
  NVIC_SetPriorityGrouping(3);
  /* enable the low frequency crystal oscillator */
  CMU_OscillatorEnable(cmuOsc_LFXO, true, true);
  /* turn on clocking of all the modules */
  CMU->HFCORECLKEN0 |= 0x0F;
  CMU->HFPERCLKEN0 |= 0xFFFF;
  /* disable clocking of the modules that are not in use */
  CMU_ClockEnable(cmuClock_AES | cmuClock_DMA | cmuClock_EBI | cmuClock_PRS, false);
  CMU_ClockEnable(cmuClock_USART0 | cmuClock_USART1 | cmuClock_USART2, false);
  CMU_ClockEnable(cmuClock_UART0 | cmuClock_ACMP0 | cmuClock_ACMP1, false);
  CMU_ClockEnable(cmuClock_DAC0 | cmuClock_ADC0 | cmuClock_I2C0 | cmuClock_VCMP, false);
  /* initialize the LCD */
  LcdInit();
  /* initialize the LED driver */
  LedInit();
  /* initialize the button driver */
  ButtonInit();
  /* initialize the buzzer driver */
  BuzzerInit();
} /*** end of HwInit ***/


/****************************************************************************************
** NAME:           Default_Handler
** PARAMETER:      none
** RETURN VALUE:   none
** DESCRIPTION:    Default Interrupt Service Routine for unused interrupts.
**
****************************************************************************************/
static void Default_Handler(void)
{
  /* unexpected interrupt occurred. hang the program for debugging purposes */
  while (1==1)
  {
    ;
  }
} /*** end of Default_Handler ***/


/****************************************************************************************
*                I N T E R R U P T     V E C T O R     T A B L E
****************************************************************************************/
__attribute__ ((section(".vectors")))
const tIsrFunc _vectors[] =
{
  { .ptr = (unsigned long)&__stack_end__ },           /* the initial stack pointer     */
  reset_handler,                                      /* the reset handler             */
  Default_Handler,                                    /* NMI Handler                   */
  Default_Handler,                                    /* Hard Fault Handler            */
  Default_Handler,                                    /* MPU Fault Handler             */
  Default_Handler,                                    /* Bus Fault Handler             */
  Default_Handler,                                    /* Usage Fault Handler           */
  Default_Handler,                                    /* Reserved                      */
  Default_Handler,                                    /* Reserved                      */
  Default_Handler,                                    /* Reserved                      */
  Default_Handler,                                    /* Reserved                      */
  vPortSVCHandler,                                    /* SVCall Handler                */
  Default_Handler,                                    /* Debug Monitor Handler         */
  Default_Handler,                                    /* Reserved                      */
  xPortPendSVHandler,                                 /* PendSV Handler                */
  xPortSysTickHandler,                                /* SysTick Handler               */
  Default_Handler,                                    /* 0 - DMA Handler               */
  Default_Handler,                                    /* 1 - GPIO_EVEN Handler         */
  Default_Handler,                                    /* 2 - TIMER0 Handler            */
  Default_Handler,                                    /* 3 - USART0_RX Handler         */
  Default_Handler,                                    /* 4 - USART0_TX Handler         */
  Default_Handler,                                    /* 5 - ACMP0 Handler             */
  Default_Handler,                                    /* 6 - ADC0 Handler              */
  Default_Handler,                                    /* 7 - DAC0 Handler              */
  Default_Handler,                                    /* 8 - I2C0 Handler              */
  Default_Handler,                                    /* 9 - GPIO_ODD Handler          */
  Default_Handler,                                    /* 10 - TIMER1 Handler           */
  Default_Handler,                                    /* 11 - TIMER2 Handler           */
  Default_Handler,                                    /* 12 - USART1_RX Handler        */
  Default_Handler,                                    /* 13 - USART1_TX Handler        */
  Default_Handler,                                    /* 14 - USART2_RX Handler        */
  Default_Handler,                                    /* 15 - USART2_TX Handler        */
  Default_Handler,                                    /* 16 - UART0_RX Handler         */
  Default_Handler,                                    /* 17 - UART0_TX Handler         */
  Default_Handler,                                    /* 18 - LEUART0 Handler          */
  Default_Handler,                                    /* 19 - LEUART1 Handler          */
  Default_Handler,                                    /* 20 - LETIMER0 Handler         */
  Default_Handler,                                    /* 21 - PCNT0 Handler            */
  Default_Handler,                                    /* 22 - PCNT1 Handler            */
  Default_Handler,                                    /* 23 - PCNT2 Handler            */
  Default_Handler,                                    /* 24 - RTC Handler              */
  Default_Handler,                                    /* 25 - CMU Handler              */
  Default_Handler,                                    /* 26 - VCMP Handler             */
  LCD_IRQHandler,                                     /* 27 - LCD Handler              */
  Default_Handler,                                    /* 28 - MSC Handler              */
  Default_Handler                                     /* 29 - AES Handler              */
};


/************************************ end of hw.c **************************************/


